'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _glMatrix = require('gl-matrix');

var _Ray = require('./math/Ray');

var _Ray2 = _interopRequireDefault(_Ray);

var _OrthographicCamera = require('./cameras/OrthographicCamera');

var _OrthographicCamera2 = _interopRequireDefault(_OrthographicCamera);

var _PerspectiveCamera = require('./cameras/PerspectiveCamera');

var _PerspectiveCamera2 = _interopRequireDefault(_PerspectiveCamera);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

/**
 * Позволяет легко определять пересечения луча с объектами.
 * Например, для определения клика пользователя.
 */

var Raycaster = function () {
    /**
     * @param {vec3} origin Точка начала луча
     * @param {vec3} direction Направление луча
     * @param {Number} [near=0] Минимальное расстояние от начала до точки пересечения
     * @param {Number} [far=Infinity] Максимальное расстояние от начала до точки пересечения
     */

    function Raycaster(origin, direction, near, far) {
        _classCallCheck(this, Raycaster);

        this.precision = 0.0001;
        this.ray = new _Ray2.default(origin, direction);
        this.near = near || 0;
        this.far = far || Infinity;
    }

    /**
     * Устанавливает начало луча в положение камеры, а направление проецирует с переданных координат
     * экрана в систему координат камеры.
     *
     * @param {vec3} coordinates
     * @param {Camera} camera
     */


    _createClass(Raycaster, [{
        key: 'setFromCamera',
        value: function setFromCamera(coordinates, camera) {
            if (camera instanceof _PerspectiveCamera2.default) {
                this.ray.origin = _glMatrix.vec3.clone(camera.position);

                var direction = _glMatrix.vec3.fromValues(coordinates[0], coordinates[1], 0.5);
                direction = camera.unproject(direction);
                _glMatrix.vec3.sub(direction, direction, camera.position);
                _glMatrix.vec3.normalize(direction, direction);
                this.ray.direction = direction;
            } else if (camera instanceof _OrthographicCamera2.default) {
                var origin = _glMatrix.vec3.fromValues(coordinates[0], coordinates[1], -1);
                this.ray.origin = camera.unproject(origin);

                this.ray.direction = _glMatrix.vec3.fromValues(0, 0, -1);

                var matrix3 = _glMatrix.mat3.create();
                _glMatrix.mat3.fromMat4(matrix3, camera.worldMatrix);
                _glMatrix.vec3.transformMat3(this.ray.direction, this.ray.direction, matrix3);
                _glMatrix.vec3.normalize(this.ray.direction, this.ray.direction);
            }
        }

        /**
         * Ищет точки пересечения луча с объектом
         * @param {Object3D} object
         * @param {Boolean} [recursive=false] Проверять ли дочерние объекты
         * @returns {Intersect[]}
         */

    }, {
        key: 'intersectObject',
        value: function intersectObject(object, recursive) {
            var intersects = [];

            object.raycast(this, intersects, recursive);

            intersects.sort(this._descSort);

            return intersects;
        }

        /**
         * Ищет точки пересечения луча с массивом объектов
         * @param {Object3D[]} objects
         * @param {Boolean} [recursive=false] Проверять ли дочерние объекты
         * @returns {Intersect[]}
         */

    }, {
        key: 'intersectObjects',
        value: function intersectObjects(objects, recursive) {
            var _this = this;

            var intersects = [];

            objects.forEach(function (obj) {
                return obj.raycast(_this, intersects, recursive);
            });

            intersects.sort(this._descSort);

            return intersects;
        }
    }, {
        key: '_descSort',
        value: function _descSort(a, b) {
            return a.distance - b.distance;
        }
    }]);

    return Raycaster;
}();

exports.default = Raycaster;

/**
 * Возвращаемое значение методов raycast. Содержит информацию о точки пересечения с объектом.
 *
 * @typedef {Object} Intersect
 * @property {Number} distance Расстояние от начала луча до точки пересечения
 * @property {vec3} point Координаты точки пересечения
 * @property {Object3D} object Объект с которым пересекся луч
 */

module.exports = exports['default'];