'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _SpriteRenderer = require('./SpriteRenderer');

var _SpriteRenderer2 = _interopRequireDefault(_SpriteRenderer);

var _TransparentRenderer = require('./TransparentRenderer');

var _TransparentRenderer2 = _interopRequireDefault(_TransparentRenderer);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

/**
 * Используется для инициализация WebGL контекста и отрисовки объектов.
 * Для некоторых объектов может использовать специфичные рендеры.
 *
 * @param {Object} options
 * @param {HTMLElement} options.container Элемент в который будет добавлен canvas
 * @param {Number} [options.pixelRatio=1] Pixel ratio экрана
 * @param {Boolean} [options.antialias=true] Использовать ли антиалиасинг
 * @param {Boolean} [options.autoClear=true] Стирать ли прошлый кадр перед новый рендерингом
 * @param {Array} [options.clearColor=true] Цвет заливки в формате RGBA
 * @param {Object} [options.sortObjects=true] Нужно ли сортировать прозрачные объекты по удаленности
 * или по renderOrder
 * */

var Renderer = function () {
    function Renderer(options) {
        _classCallCheck(this, Renderer);

        this._container = typeof options.container === 'string' ? document.getElementById(options.container) : options.container;

        this._pixelRatio = options.pixelRatio || 1;
        this._antialias = options.antialias !== undefined ? options.antialias : true;

        /**
         * Определяет стирать ли прошлый кадр перед новый рендерингом
         * @type {Boolean}
         */
        this.autoClear = options.autoClear !== undefined ? options.autoClear : true;

        /**
         * Цвет заливки в формате RGBA
         * @type {Array}
         */
        this.clearColor = options.clearColor || [1, 1, 1, 1];
        this.sortObjects = true;

        this._spriteRenderer = new _SpriteRenderer2.default();
        this._transparentRenderer = new _TransparentRenderer2.default();

        this._initializeCanvas();
    }

    /**
     * Устанавливает параметр pixel ratio
     * @param {Number} value
     */


    _createClass(Renderer, [{
        key: 'setPixelRatio',
        value: function setPixelRatio(value) {
            this._pixelRatio = value;

            return this;
        }

        /**
         * Возвращает текущий pixel ratio
         * @returns {Number}
         */

    }, {
        key: 'getPixelRatio',
        value: function getPixelRatio() {
            return this._pixelRatio;
        }

        /**
         * Устанавливает размеры элементу canvas и viewport для WebGL
         * @param {Number} width Ширина в пикселях
         * @param {Number} height Высота в пикселях
         */

    }, {
        key: 'setSize',
        value: function setSize(width, height) {
            this._size = [width * this._pixelRatio, height * this._pixelRatio];

            this._canvasElement.width = this._size[0];
            this._canvasElement.height = this._size[1];
            this._canvasElement.style.width = width + 'px';
            this._canvasElement.style.height = height + 'px';
            this._gl.viewport(0, 0, this._size[0], this._size[1]);

            return this;
        }

        /**
         * Возвращает текущий viewport WebGL
         * @returns {Array}
         */

    }, {
        key: 'getSize',
        value: function getSize() {
            return this._size;
        }

        /**
         * Очищает текущий кадр и заливает цветом указанным в clearColor
         */

    }, {
        key: 'clear',
        value: function clear() {
            var gl = this._gl;

            gl.clearColor.apply(gl, this.clearColor);
            gl.clear(gl.COLOR_BUFFER_BIT | gl.DEPTH_BUFFER_BIT);

            return this;
        }

        /**
         * Рисует сцену
         * @param {Scene} scene Сцена
         * @param {Camera} camera Камера
         */

    }, {
        key: 'render',
        value: function render(scene, camera) {
            var gl = this._gl;
            var typedObjects = {
                common: [],
                transparent: [],
                sprites: []
            };

            scene.typifyForRender(typedObjects);

            gl.clearDepth(1);
            gl.clearStencil(0);

            gl.enable(gl.DEPTH_TEST);
            gl.depthFunc(gl.LEQUAL);

            gl.frontFace(gl.CCW);
            gl.cullFace(gl.BACK);
            gl.enable(gl.CULL_FACE);

            if (this.autoClear) {
                this.clear();
            }

            camera.updateLocalMatrix();
            camera.updateWorldMatrix();

            gl.disable(gl.BLEND);

            var state = {
                renderer: this,
                scene: scene,
                camera: camera,
                gl: gl
            };
            // TODO: make state immutable?

            typedObjects.common.forEach(function (object) {
                return object.render(state);
            });

            this._transparentRenderer.render(state, typedObjects.transparent);

            this._spriteRenderer.render(state, typedObjects.sprites);

            return this;
        }
    }, {
        key: '_initializeCanvas',
        value: function _initializeCanvas() {
            this._canvasElement = document.createElement('canvas');
            this._container.appendChild(this._canvasElement);

            var attributes = {
                antialias: this._antialias
            };

            this._gl = this._canvasElement.getContext('webgl', attributes) || this._canvasElement.getContext('experimental-webgl', attributes);
        }
    }]);

    return Renderer;
}();

exports.default = Renderer;

/**
 * Состояние рендера. Передается объектам для отрисовки.
 *
 * @typedef {Object} State
 * @property {WebGLRenderingContext} gl
 * @property {Scene} scene
 * @property {Camera} camera
 * @property {Renderer} renderer
 */

/**
 * Объект используется для распределения объектов по типам рендеров,
 * т.к. прозрачные объекты и спрайты рендерятся отдельно.
 *
 * @typedef {Object} TypedObjects
 * @property {Array} common Сюда складываются все объекты, для которых нет специальных рендеров
 * @property {Array} transparent Прозрачные объекты
 * @property {Array} sprites Спрайты
 */

module.exports = exports['default'];